@@ -1,7 +1,7 @@
 #[cfg(test)]
 mod test {
     use rabuf::BufFile;
-    use std::fs::File;
+    use std::fs::OpenOptions;
     use std::io::{Read, Seek, SeekFrom, Write};
     //
     macro_rules! base_dir {
@@ -17,7 +17,13 @@ mod test {
         let path = concat!(base_dir!(), "/test_1");
         let bw = b"ABCEDFG\nhijklmn\n";
         //
-        let f = File::create(path).unwrap();
+        //let f = File::create(path).unwrap();
+        let f = OpenOptions::new()
+            .create(true)
+            .read(true)
+            .write(true)
+            .open(path)
+            .unwrap();
         let mut bf = BufFile::new("tes", f).unwrap();
         bf.write_all(bw).unwrap();
         //
@@ -34,12 +40,24 @@ mod test {
         //
         let bw = b"abcdefg\nHIJKLMN\n";
         {
-            let f = File::create(path).unwrap();
+            //let f = File::create(path).unwrap();
+            let f = OpenOptions::new()
+                .create(true)
+                .read(true)
+                .write(true)
+                .open(path)
+                .unwrap();
             let mut bf = BufFile::new("tes", f).unwrap();
             bf.write_all(bw).unwrap();
         }
         {
-            let f = File::open(path).unwrap();
+            //let f = File::open(path).unwrap();
+            let f = OpenOptions::new()
+                .create(true)
+                .read(true)
+                .write(true)
+                .open(path)
+                .unwrap();
             let mut bf = BufFile::new("tes", f).unwrap();
             let mut br = vec![0u8; bw.len()];
             bf.read_exact(&mut br).unwrap();
@@ -53,12 +71,24 @@ mod test {
         //
         let bw = b"1234567\nABCDEFG\n8901234\nabcdefg\n";
         {
-            let f = File::create(path).unwrap();
+            //let f = File::create(path).unwrap();
+            let f = OpenOptions::new()
+                .create(true)
+                .read(true)
+                .write(true)
+                .open(path)
+                .unwrap();
             let mut bf = BufFile::with_capacity("tes", f, 2, 4).unwrap();
             bf.write_all(bw).unwrap();
         }
         {
-            let f = File::open(path).unwrap();
+            //let f = File::open(path).unwrap();
+            let f = OpenOptions::new()
+                .create(true)
+                .read(true)
+                .write(true)
+                .open(path)
+                .unwrap();
             let mut bf = BufFile::new("tes", f).unwrap();
             let mut br = vec![0u8; bw.len()];
             bf.read_exact(&mut br).unwrap();
@@ -71,19 +101,32 @@ mod test {
         let path = concat!(base_dir!(), "/test_seek_over_the_end");
         //
         let bw = b"abcdefg\n";
-        {
-            let f = File::create(path).unwrap();
+        let pos = {
+            //let f = File::create(path).unwrap();
+            let f = OpenOptions::new()
+                .create(true)
+                .read(true)
+                .write(true)
+                .open(path)
+                .unwrap();
             let mut bf = BufFile::with_capacity("tes", f, 2, 4).unwrap();
             bf.seek(SeekFrom::End(0)).unwrap();
             // test a sparse file
-            bf.seek(SeekFrom::Current(16)).unwrap();
+            let pos = bf.seek(SeekFrom::Current(16)).unwrap();
             bf.write_all(bw).unwrap();
-        }
+            pos
+        };
         {
-            let f = File::open(path).unwrap();
+            //let f = File::open(path).unwrap();
+            let f = OpenOptions::new()
+                .create(true)
+                .read(true)
+                .write(true)
+                .open(path)
+                .unwrap();
             let mut bf = BufFile::new("tes", f).unwrap();
             let mut br = vec![0u8; bw.len()];
-            bf.seek(SeekFrom::Start(16)).unwrap();
+            bf.seek(SeekFrom::Start(pos)).unwrap();
             bf.read_exact(&mut br).unwrap();
             assert_eq!(&br, bw);
         }
